@@ -147,6 +147,9 @@ for($i=0;$i<$nbr_tables;$i++){
 				$a_extra = $a["Extra"];
 				$a_type = $a["Type"];
 				$a_collate = $a["Collation"];
+				if($a_collate == "utf8mb3_unicode_ci"){
+					$a_collate = "utf8_unicode_ci";
+				}
 				switch($a_type){
 				case "blob":
 					$type = $a_type;
@@ -157,6 +160,9 @@ for($i=0;$i<$nbr_tables;$i++){
                         		$r2 = mysqli_query($con, $q2)or die("Cannot execute query: \"$q2\" line ".__LINE__." in file ".__FILE__.", mysql said: ".mysqli_error($con));
 					$a2 = mysqli_fetch_array($r2);
 					if($a2["character_set_name"] != 'latin1'){
+						if($a2["character_set_name"] == "utf8mb3"){
+							$a2["character_set_name"] = "utf8";
+						}
 						$type .= ' character set '.$a2["character_set_name"];
 					} 
 					mysqli_free_result($r2);
@@ -226,7 +232,7 @@ for($i=0;$i<$nbr_tables;$i++){
 				if(!findKeyInTable($con, $curtbl,$key_name)){
 					$var_2_add = "UNIQUE KEY ".$key_name;
 					$q = "ALTER TABLE ".$curtbl." ADD $var_2_add ".$keys[$key_name].";";
-					$r = mysqli_query($q)or die("\nCannot execute query: \"$q\" line ".__LINE__." in file ".__FILE__.", mysql said: ".mysqli_error($con));
+					$r = mysqli_query($con, $q)or die("\nCannot execute query: \"$q\" line ".__LINE__." in file ".__FILE__.", mysql said: ".mysqli_error($con));
 				}
 			}
 		}
@@ -358,21 +364,25 @@ function check_configurable_variables($con, $table, $items_json_array){
 			$a_def = $af["Default"];
 			switch($attr["type"]){
 			case "int":
-				if($a_type != "int(11)" || $a_def != $default){
+				if($a_type != "bigint(20)" || $a_def != $default){
+					echo "$a_type != bigint or $a_def != $default \n";
 					$qfix = "ALTER TABLE $table CHANGE $col $col bigint NOT NULL default '$default'";
 					$rfix = mysqli_query($con, $qfix)or die("Cannot execute query: \"$qfix\" line ".__LINE__." in file ".__FILE__.", mysql said: ".mysqli_error($con));
+                                        echo "fix ";
 				}
 				break;
 			case "float":
 				if($a_type != "float(11)" || $a_def != $default){
 					$qfix = "ALTER TABLE $table CHANGE $col $col float(11) NOT NULL default '$default'";
 					$rfix = mysqli_query($con, $qfix)or die("Cannot execute query: \"$qfix\" line ".__LINE__." in file ".__FILE__.", mysql said: ".mysqli_error($con));
+                                        echo "fix ";
 				}
 				break;
 			case "boolean":
 				if($a_type != "enum('yes','no')" || $a_def != $default){
 					$qfix = "ALTER TABLE $table CHANGE $col $col enum('yes','no') NOT NULL default '$default'";
 					$rfix = mysqli_query($con, $qfix)or die("Cannot execute query: \"$qfix\" line ".__LINE__." in file ".__FILE__.", mysql said: ".mysqli_error($con));
+                                        echo "fix ";
 				}
 				break;
 			case "fqdn":
@@ -380,12 +390,13 @@ function check_configurable_variables($con, $table, $items_json_array){
 				if($a_type != "varchar(255)" || $a_def != $default){
 					$qadd = "ALTER TABLE $table CHANGE $col $col varchar(255) NOT NULL default '$default'";
 					$radd = mysqli_query($con, $qadd)or die("Cannot execute query: \"$qadd\" line ".__LINE__." in file ".__FILE__.", mysql said: ".mysqli_error($con));
+                                        echo "fix ";
 				}
 				break;
 			default:
 				break;
 			}
-			echo "fix $col ";
+			echo "$col ";
 		}
 	}
 	echo "\n";
@@ -421,6 +432,9 @@ $q[] = "INSERT IGNORE INTO `roles` (`id`, `name`) VALUES (14,'tempest');";
 $q[] = "INSERT IGNORE INTO `roles` (`id`, `name`) VALUES (15,'billosd');";
 $q[] = "INSERT IGNORE INTO `roles` (`id`, `name`) VALUES (16,'billmon');";
 $q[] = "INSERT IGNORE INTO `roles` (`id`, `name`) VALUES (17,'dns');";
+$q[] = "INSERT IGNORE INTO `roles` (`id`, `name`) VALUES (18,'metal');";
+$q[] = "INSERT IGNORE INTO `roles` (`id`, `name`) VALUES (19,'radosgw');";
+$q[] = "INSERT IGNORE INTO `roles` (`id`, `name`) VALUES (20,'sqlmsg');";
 
 foreach ($q as $sql){
 	$r = mysqli_query($con, $sql);
