@@ -251,6 +251,8 @@ if($n == 0){
 
     if (isset($data["phys-blockdevices"])){
         foreach($data["phys-blockdevices"] as &$blockdevice){
+            $blk_ctrl   = $blockdevice['controller'];
+            $blk_enc    = $blockdevice['enclosure'];
             $blk_slot   = $blockdevice['slot'];
             $blk_model  = $blockdevice['model'];
             $blk_serial = $blockdevice['serial'];
@@ -259,13 +261,15 @@ if($n == 0){
 
             $reg_alpha = '/^[.0-9a-zA-Z- \(\)\[\]:\\/\\\\]{1,64}$/';
             $reg_int = '/^[.0-9]{1,64}$/';
+            if(!preg_match($reg_int,   $blk_ctrl))   die("Block device controller suspicious line ".__LINE__." file ".__FILE__);
+            if(!preg_match($reg_int,   $blk_enc))    die("Block device enclosure suspicious line ".__LINE__." file ".__FILE__);
             if(!preg_match($reg_int,   $blk_slot))   die("Block device slot suspicious line ".__LINE__." file ".__FILE__);
             if(!preg_match($reg_alpha, $blk_model))  die("Block device model suspicious line ".__LINE__." file ".__FILE__);
             if(!preg_match($reg_alpha, $blk_serial)) die("Block device serial suspicious line ".__LINE__." file ".__FILE__);
             if(!preg_match($reg_int,   $blk_size))   die("Block device size suspicious line".__LINE__." file ".__FILE__);
             if(!preg_match($reg_alpha, $blk_state))  die("Block device state suspicious line ".__LINE__." file ".__FILE__);
             $safe_blk_size = $blk_size / (1024*1024);
-            $r = mysqli_query($con, "INSERT INTO physblockdevices (machine_id, slot, model, serial, size, state) VALUES ('".$machine_id."', '".$blk_slot."', '".$blk_model."', '".$blk_serial."', '".$blk_size."', '".$blk_state."')");
+            $r = mysqli_query($con, "INSERT INTO physblockdevices (machine_id, controller, enclosure, slot, model, serial, size, state) VALUES ('".$machine_id."', '".$blk_ctrl."', '".$blk_enc."', '".$blk_slot."', '".$blk_model."', '".$blk_serial."', '".$blk_size."', '".$blk_state."')");
         }
     }
 
@@ -285,7 +289,7 @@ if($n == 0){
                 if(!preg_match($reg_alpha, $blkdev_ctrl_product))          die("Block device controller product suspicious line ".__LINE__." file ".__FILE__);
                 if(!preg_match($reg_alpha, $blkdev_ctrl_ctrl_type))        die("Block device controller ctrl type suspicious line ".__LINE__." file ".__FILE__);
                 if(!preg_match($reg_int,   $blkdev_ctrl_firmware_version)) die("Block device controller fw version suspicious line ".__LINE__." file ".__FILE__);
-                $r = mysqli_query($con, "INSERT INTO physblockdevices (machine_id, hwid, vendor, product, ctrl_type, firmware_version) VALUES ('".$machine_id."', '".$blkdev_ctrl_hwid."', '".$blkdev_ctrl_vendor."', '".$blkdev_ctrl_product."', '".$blkdev_ctrl_ctrl_type."', '".$blkdev_ctrl_firmware_version."')");
+                $r = mysqli_query($con, "INSERT INTO blkdev_ctrl (machine_id, hwid, vendor, product, ctrl_type, firmware_version) VALUES ('".$machine_id."', '".$blkdev_ctrl_hwid."', '".$blkdev_ctrl_vendor."', '".$blkdev_ctrl_product."', '".$blkdev_ctrl_ctrl_type."', '".$blkdev_ctrl_firmware_version."')");
             }
         }
     }
@@ -427,14 +431,22 @@ if($n == 0){
     if (isset($data["phys-blockdevices"])){
         $blkdev_array = array();
         foreach($data["phys-blockdevices"] as &$blockdevice){
+            $blk_ctrl   = $blockdevice['controller'];
+            $blk_enc    = $blockdevice['enclosure'];
             $blk_slot   = $blockdevice['slot'];
             $blk_model  = $blockdevice['model'];
             $blk_serial = $blockdevice['serial'];
             $blk_size   = $blockdevice['size'];
             $blk_state  = $blockdevice['state'];
 
-            $reg_alpha = '/^[.0-9a-zA-Z- \(\)]{1,64}$/';
+            $reg_alpha = '/^[.0-9a-zA-Z- \(\)\[\]]{1,64}$/';
             $reg_int = '/^[.0-9]{1,64}$/';
+            if($blk_ctrl != ""){
+                if(!preg_match($reg_alpha, $blk_ctrl))   die("Block device controller suspicious line ".__LINE__." file ".__FILE__);
+            }
+            if($blk_enc != ""){
+                if(!preg_match($reg_int,   $blk_enc))    die("Block device enclosure suspicious line ".__LINE__." file ".__FILE__);
+            }
             if(!preg_match($reg_int,   $blk_slot))   die("Block device slot suspicious line ".__LINE__." file ".__FILE__);
             if(!preg_match($reg_alpha, $blk_model))  die("Block device model suspicious line ".__LINE__." file ".__FILE__);
             if(!preg_match($reg_alpha, $blk_serial)) die("Block device serial suspicious line ".__LINE__." file ".__FILE__);
@@ -443,11 +455,13 @@ if($n == 0){
             $safe_blk_size = $blk_size / (1024*1024);
 
             $blkdev_array[] = array(
-                "slot"   => $blk_slot,
-                "model"  => $blk_model,
-                "serial" => $blk_serial,
-                "size"   => $blk_size,
-                "state"  => $blk_state,
+                "controller" => $blk_ctrl,
+                "enclosure"  => $blk_enc,
+                "slot"       => $blk_slot,
+                "model"      => $blk_model,
+                "serial"     => $blk_serial,
+                "size"       => $blk_size,
+                "state"      => $blk_state,
                 );
         }
 
@@ -455,12 +469,14 @@ if($n == 0){
         $all_if_in_sql = "yes";
         for($i=0;$i<sizeof($blkdev_array);$i++){
             $myblkdev = $blkdev_array[$i];
-            $safe_blk_slot   = $myblkdev["slot"];
-            $safe_blk_model  = $myblkdev["model"];
-            $safe_blk_serial = $myblkdev["serial"];
-            $safe_blk_size   = $myblkdev["size"];
-            $safe_blk_state  = $myblkdev["state"];
-            $q = "SELECT id FROM physblockdevices WHERE machine_id='$machine_id' AND slot='$safe_blk_slot' AND model='$safe_blk_model' AND serial='$safe_blk_serial' AND size='$safe_blk_size' AND state='$safe_blk_state'";
+            $safe_blk_controller = $myblkdev["controller"];
+            $safe_blk_enclosure  = $myblkdev["enclosure"];
+            $safe_blk_slot       = $myblkdev["slot"];
+            $safe_blk_model      = $myblkdev["model"];
+            $safe_blk_serial     = $myblkdev["serial"];
+            $safe_blk_size       = $myblkdev["size"];
+            $safe_blk_state      = $myblkdev["state"];
+            $q = "SELECT id FROM physblockdevices WHERE machine_id='$machine_id' AND controller='$safe_blk_controller' AND enclosure='$safe_blk_enclosure' AND slot='$safe_blk_slot' AND model='$safe_blk_model' AND serial='$safe_blk_serial' AND size='$safe_blk_size' AND state='$safe_blk_state'";
             $r = mysqli_query($con, $q);
             $n = mysqli_num_rows($r);
             if($n != 1){
@@ -481,12 +497,14 @@ if($n == 0){
             $r = mysqli_query($con, "DELETE FROM physblockdevices WHERE machine_id='".$machine_id."'");
             for($i=0;$i<sizeof($blkdev_array);$i++){
                 $myblkdev = $blkdev_array[$i];
-                $safe_blk_slot   = $myblkdev["slot"];
-                $safe_blk_model  = $myblkdev["model"];
-                $safe_blk_serial = $myblkdev["serial"];
-                $safe_blk_size   = $myblkdev["size"];
-                $safe_blk_state  = $myblkdev["state"];
-                $r = mysqli_query($con, "INSERT INTO physblockdevices (machine_id, slot, model, serial, size, state) VALUES ('".$machine_id."', '".$safe_blk_slot."', '".$safe_blk_model."', '".$safe_blk_serial."', '".$safe_blk_size."', '".$safe_blk_state."')");
+                $safe_blk_controller = $myblkdev["controller"];
+                $safe_blk_enclosure  = $myblkdev["enclosure"];
+                $safe_blk_slot       = $myblkdev["slot"];
+                $safe_blk_model      = $myblkdev["model"];
+                $safe_blk_serial     = $myblkdev["serial"];
+                $safe_blk_size       = $myblkdev["size"];
+                $safe_blk_state      = $myblkdev["state"];
+                $r = mysqli_query($con, "INSERT INTO physblockdevices (machine_id, controller, enclosure, slot, model, serial, size, state) VALUES ('".$machine_id."', '".$safe_blk_controller."', '".$safe_blk_enclosure."', '".$safe_blk_slot."', '".$safe_blk_model."', '".$safe_blk_serial."', '".$safe_blk_size."', '".$safe_blk_state."')");
             }
         }
     }
@@ -507,7 +525,9 @@ if($n == 0){
                 if(!preg_match($reg_alpha, $blkdev_ctrl_vendor))           die("Block device controller vendor suspicious line ".__LINE__." file ".__FILE__);
                 if(!preg_match($reg_alpha, $blkdev_ctrl_product))          die("Block device controller product suspicious: $blkdev_ctrl_product line ".__LINE__." file ".__FILE__);
                 if(!preg_match($reg_alpha, $blkdev_ctrl_ctrl_type))        die("Block device controller ctrl type suspicious line ".__LINE__." file ".__FILE__);
-                if(!preg_match($reg_int,   $blkdev_ctrl_firmware_version)) die("Block device controller fw version suspicious line ".__LINE__." file ".__FILE__);
+                if($blkdev_ctrl_firmware_version != ""){
+                    if(!preg_match($reg_int,   $blkdev_ctrl_firmware_version)) die("Block device controller fw version suspicious line ".__LINE__." file ".__FILE__);
+                }
 
                 $blkdev_ctrl_array[] = array(
                     "hwid" => $blkdev_ctrl_hwid,
@@ -565,33 +585,33 @@ if($n == 0){
             if(isset($iface["name"]) && isset($iface["macaddr"]) && isset($iface["max_speed"])){
                 $if_name = $iface["name"];
                 $reg = '/^[0-9a-zA-Z-]{1,64}$/';
-                if(!preg_match($reg,$if_name)) die("Network interface device name suspicious line ".__LINE__." file ".__FILE__);
+                if(!preg_match($reg,$if_name)) die("Network interface device name '$if_name' suspicious line ".__LINE__." file ".__FILE__);
                 $safe_if_name = $if_name;
 
                 $if_macaddr = $iface["macaddr"];
                 $reg = '/^[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}$/';
-                if(!preg_match($reg,$if_macaddr)) die("Network interface MAC address suspicious line ".__LINE__." file ".__FILE__);
+                if(!preg_match($reg,$if_macaddr)) die("Network interface MAC address '$if_macaddr' suspicious line ".__LINE__." file ".__FILE__);
                 $safe_if_macaddr = $if_macaddr;
 
                 $if_max_speed = $iface["max_speed"];
                 $reg = '/^[0-9]{1,10}$/';
-                if(!preg_match($reg,$if_max_speed)) die("Network interface max_speed suspicious line ".__LINE__." file ".__FILE__);
+                if(!preg_match($reg,$if_max_speed)) die("Network interface max_speed '$if_max_speed' suspicious line ".__LINE__." file ".__FILE__);
                 $safe_if_max_speed = $if_max_speed;
 
                 $if_switch_hostname = $iface["switch_hostname"];
                 $reg = '/^((?!-))(xn--)?[a-z0-9][.a-z0-9-]{0,61}[a-z0-9]{0,1}$/';
-                if(!preg_match($reg,$if_switch_hostname)) die("Switch hostname suspicious line ".__LINE__." file ".__FILE__);
+                if(!preg_match($reg,$if_switch_hostname)) die("Switch hostname '$if_switch_hostname' suspicious line ".__LINE__." file ".__FILE__);
                 $safe_if_switch_hostname = $if_switch_hostname;
 
                 $if_switchport_name = $iface["switchport_name"];
-                $reg = '/^[.:0-9a-zA-Z-]{1,64}$/';
-                if(!preg_match($reg,$if_switchport_name)) die("Switchport name suspicious line ".__LINE__." file ".__FILE__);
+                $reg = '/^[\/.:0-9a-zA-Z-]{1,64}$/';
+                if(!preg_match($reg,$if_switchport_name)) die("Switchport name '$if_switchport_name' suspicious line ".__LINE__." file ".__FILE__);
                 $safe_if_switchport_name = $if_switchport_name;
 
                 if(isset($iface["firmware_version"]) && $iface["firmware_version"] != ""){
                     $if_firmware_version = $iface["firmware_version"];
-                    $reg = '/^[.:0-9a-zA-Z-]{1,64}$/';
-                    if(!preg_match($reg,$if_firmware_version)) die("Iface firmware version suspicious line ".__LINE__." file ".__FILE__);
+                    $reg = '/^[\/.:0-9a-zA-Z-]{1,64}$/';
+                    if(!preg_match($reg,$if_firmware_version)) die("Iface firmware version '$if_firmware_version' suspicious line ".__LINE__." file ".__FILE__);
                     $safe_if_firmware_version = $if_firmware_version;
                 }else{
                     $safe_if_firmware_version = "";
@@ -600,7 +620,7 @@ if($n == 0){
                 if(isset($iface["driver"]) && $iface["driver"] != ""){
                     $if_driver = $iface["driver"];
                     $reg = '/^[_.:0-9a-zA-Z-]{1,64}$/';
-                    if(!preg_match($reg,$if_driver)) die("Iface driver name suspicious line ".__LINE__." file ".__FILE__);
+                    if(!preg_match($reg,$if_driver)) die("Iface driver name '$if_driver' suspicious line ".__LINE__." file ".__FILE__);
                     $safe_if_driver = $if_driver;
                 }else{
                     $safe_if_driver = "";
@@ -669,6 +689,17 @@ if($n == 0){
         }
     }
 }
+
+# Refretch the $machine, as it may have changed above.
+$q = "SELECT * FROM machines WHERE id='$machine_id'";
+$r = mysqli_query($con, $q);
+$n = mysqli_num_rows($r);
+if($n != 1){
+    die("Cannot fetch machine $machine_id to perform automations: ".__FILE__." line ".__LINE__);
+}
+$machine = mysqli_fetch_array($r);
+
+# Apply the automations
 if(isset($conf["ipmi"]["automatic_ipmi_numbering"]) && $conf["ipmi"]["automatic_ipmi_numbering"]){
     automatic_ipmi_assign($con, $conf, $machine_id);
 }
@@ -681,25 +712,41 @@ if(isset($conf["auto_racking"]["auto_rack_machines_info"]) && $conf["auto_rackin
 
 if(isset($conf["megacli"]["megacli_auto_clear"]) && $conf["megacli"]["megacli_auto_clear"]){
     if($conf["megacli"]["megacli_auto_clear_num_of_discovery"] == $report_counter){
-        hardware_profile_apply_megacli_reset_raid($con, $conf, $machine_id);
+        if($machine["status"] == "live"){
+            hardware_profile_apply_megacli_reset_raid($con, $conf, $machine_id);
+        }else{
+            print("Skipping RAID reset as server is not in status==live\n");
+        }
     }
 }
 
 if(isset($conf["megacli"]["megacli_auto_apply"]) && $conf["megacli"]["megacli_auto_apply"]){
     if($conf["megacli"]["megacli_auto_apply_num_of_discovery"] == $report_counter){
-        hardware_profile_apply_megacli_raid($con, $conf, $machine_id, "");
+        if($machine["status"] == "live"){
+            hardware_profile_apply_megacli_raid($con, $conf, $machine_id, "");
+        }else{
+            print("Skipping RAID config as server is not in status==live\n");
+        }
     }
 }
 
 if(isset($conf["auto_provision"]["auto_add_machines_to_cluster"]) && $conf["auto_provision"]["auto_add_machines_to_cluster"]){
     if($conf["auto_provision"]["auto_add_machines_num_of_discovery"] == $report_counter){
-        $json = auto_provision_add_machine($con, $conf, $machine_id);
+        if($machine["status"] == "live"){
+            $json = auto_provision_add_machine($con, $conf, $machine_id);
+        }else{
+            print("Skipping auto-add, as server is not in status==live\n");
+        }
     }
 }
 
 if(isset($conf["auto_install_os"]["auto_install_machines_os"]) && $conf["auto_install_os"]["auto_install_machines_os"]){
     if($conf["auto_install_os"]["auto_install_machines_num_of_discovery"] == $report_counter){
-        oci_install_machine($con, $conf, $machine_id);
+        if($machine["status"] == "live"){
+            oci_install_machine($con, $conf, $machine_id);
+        }else{
+            print("Skipping auto-install, as server is not in status==live\n");
+        }
     }
 }
 
